`timescale 1ns / 1ps
///////////////////////////////////////////////////////////////////////////////////////
// Company             : UEC
// Engineer            : 
// 
// Create Date         : August/12/2014 
// Module Name         : sakura_w_vcp_mode
// Project Name        : sakura_w_vcp_mode
// Target Devices      : xc6slx9-2csg225
// Tool versions       : 14.6
// Description         : 
//
// Dependencies        : 
//
// Version             : 1.0
// Last Uodate         : August/12/2014
// Additional Comments : 
///////////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Satoh LaboratoryCUEC

module sakura_w_vcp_mode (
  input        RSTn,          // Power on reset input
  input        CLK,           // 48MHz input

  // USB vcp interface
  input        USB_RXD,       // Receive data
  output reg   USB_TXD,       // Transmit data
  input        USB_RTSn,      // Ready to send handshake input
  output       USB_CTSn,      // Clear to send handshake output
  input        FTDI_PWREN_B,
  input        FTDI_TXDEN,

  // Smart card interface
  output reg   CARD_RST,      // Smart CARD reset
  output       CARD_CLK,      // Smart CARD clock
  inout        CARD_IO,       // Smart CARD serial IO
  input        CARD_AUX1,
  input        CARD_AUX2,

  input        CARD_DETECT,
  output reg   CARD_POWEN,
   
  // SAKURA-G LED, header pin, push switch, dip switch
  output [9:0] LED,
  //  input  [1:0] DIP_SW,
  input        PUSH_SW,
  output       gpio_txd, gpio_rxd, gpio_TRG0, gpio_TRG1,

  // SAKURA-W LED, header pin, dip switch
  output [3:0] W_LED_OUT
//  input  [3:0] W_DSW_IN
);

// ==========================================================================
// Internal signals
// ==========================================================================
  // Internal clock
  wire         buff_clk;               // Bufferd input clock 
  wire         fbclk_out;              // PLL feedback clock output
  wire         buff_fbclk;             // Bufferd PLL feedback clock
  wire         clk_out0;
  wire         clock;
  // Reset
  reg  [15:0]  rst_count;              // Delay reset counter
  reg          delay_rstn;             // 65,537 clock delay reset
  wire         resetn;                 // Internal delay

  // FT2232H
  reg          usb_rxdata;

  // Smartard intarface
  reg          sc_txdata;

// ==========================================================================
// Equetions
// ==========================================================================
  // Clock input buffer
  IBUFG u0 (.I( CLK ), .O( buff_clk ));

  // ------------------------------------------------------------------------
  // Clock generater
  // input  clock: 48 MHz
  // output clock: 3.571 MHz (48MHz * 9 / 121)
  // ------------------------------------------------------------------------
  PLL_BASE #(.BANDWIDTH          ("OPTIMIZED"),
             .CLK_FEEDBACK       ("CLKFBOUT"),
             .COMPENSATION       ("SYSTEM_SYNCHRONOUS"),
             .DIVCLK_DIVIDE      (1),
             .CLKFBOUT_MULT      (9),
             .CLKFBOUT_PHASE     (0.000),
             .CLKOUT0_DIVIDE     (121),
             .CLKOUT0_PHASE      (0.000),
             .CLKOUT0_DUTY_CYCLE (0.500),
             .CLKIN_PERIOD       (20.833),
             .REF_JITTER         (0.010))
  pll_base ( .CLKIN( buff_clk ), .RST( ~RSTn ), 
             .CLKFBIN( buff_fbclk ), .LOCKED( locked ),
             .CLKFBOUT( fbclk_out ),
             .CLKOUT0( clk_out0 ),.CLKOUT1(), .CLKOUT2(), .CLKOUT3(), .CLKOUT4(), .CLKOUT5()
           );
   
  BUFG u1 (.I( fbclk_out ), .O( buff_fbclk ));
  BUFG u2 (.I( clk_out0 ),  .O( clock ));

  // ------------------------------------------------------------------------
  // Reset
  // RSTn -> locked -> delay_rstn ---> resetn
  //                      PUSH_SW -|
  // ------------------------------------------------------------------------
  always @( posedge clock or negedge locked ) begin
    if ( locked == 1'b0 ) begin
      rst_count <= 16'h0000;
      delay_rstn <= 1'b0;
    end
    else begin
      if ( ~&rst_count ) rst_count <= rst_count + 1'b1;
      else rst_count <= rst_count;

      delay_rstn <= &rst_count;
    end
  end

  assign resetn = ~( ~delay_rstn | PUSH_SW | ~USB_RTSn );


  // ------------------------------------------------------------------------
  // FT2232H Interface
  // ------------------------------------------------------------------------
   // Masking rxd ECHO
	reg  [5:0] mask;
	
   always @(posedge clock or negedge resetn)
     if (resetn == 1'b0)        mask <= 6'b000000;
     else if (~sc_txdata ) mask <= 6'b111111;
     else            mask <= {1'b0, mask[5:1]};


  always @( posedge clock or negedge resetn ) begin
    if ( resetn == 1'b0 ) begin
      USB_TXD  <= 1'b1;
    end
    else begin
      USB_TXD  <= ( mask[0] == 1'b1 )? 1'b1 : CARD_IO;
//      USB_TXD  <= ( FTDI_TXDEN == 1'b1 )? 1'b1 : CARD_IO;
    end
  end


  assign USB_CTSn = CARD_DETECT;

  // ------------------------------------------------------------------------
  // Smart card interface
  // ------------------------------------------------------------------------
  always @( posedge clock or negedge RSTn ) begin
    if ( RSTn == 1'b0 ) begin
      sc_txdata <= 1'b1;
	   CARD_POWEN <= 1'b0;
      CARD_RST   <= 1'b0;
    end
    else begin
      sc_txdata <= USB_RXD;
	   CARD_POWEN <= ~CARD_DETECT;
      CARD_RST   <= resetn;
    end
  end

//  always @( posedge clock ) begin
//    sc_txdata <= USB_RXD;//usb_rxdata;
//    CARD_POWEN <= ~CARD_DETECT;
//    CARD_RST   <= resetn;
//  end



  // Smart card clock driver.
  // 3.571 MHz output
  ODDR2 u4 (.D0( 1'b1 ), .D1( 1'b0 ), .C0( clock ), .C1( ~clock ),
            .Q( CARD_CLK ), .CE( 1'b1 ), .R( 1'b0 ), .S( 1'b0 ));

  // Sirial data io
  assign CARD_IO = ( sc_txdata == 1'b0 )? 1'b0 : 1'bz;

  // ------------------------------------------------------------------------
  // LED Display
  // ------------------------------------------------------------------------
  // SAKURA-G on-board LED
  assign LED[9] = FTDI_TXDEN;
  assign LED[8] = FTDI_PWREN_B;
  assign LED[7] = ~resetn;
  assign LED[6] = ~RSTn;
  assign LED[5] = ~USB_RTSn;
  assign LED[4] =  PUSH_SW;
  assign LED[3] = ~CARD_DETECT;
  assign LED[2] = ~CARD_IO;
  assign LED[1] = ~USB_RXD;
  assign LED[0] = ~USB_TXD;

  // SAKURA-W on-board LED
//  assign W_LED_OUT = W_DSW_IN;
  assign W_LED_OUT[0] = CARD_POWEN;
  assign W_LED_OUT[1] = CARD_RST;
  assign W_LED_OUT[2] = CARD_DETECT;
  assign W_LED_OUT[3] = CARD_AUX1;
 
  // ------------------------------------------------------------------------
  // GPIO
  // ------------------------------------------------------------------------
  assign gpio_txd  = sc_txdata;
  assign gpio_rxd  = USB_RXD;
  assign gpio_TRG0 = CARD_AUX1;
  assign gpio_TRG1 = CARD_AUX2;
   
endmodule
